@@ -7,7 +7,7 @@ use crate::{
     app::AppLifeCycle,
     ecs::{
         components::NonPersistent,
-        pipeline::{PipelineEngine, PipelineID},
+        pipeline::{PipelineEngine, PipelineId},
     },
     state::{State, StateChange, StateToken},
 };
@@ -23,20 +23,20 @@ use std::{
 use typid::ID;
 
 pub type System = fn(&mut Universe);
-pub type UniverseID = ID<Universe>;
+pub type UniverseId = ID<Universe>;
 pub type Resource = dyn Any + Send + Sync;
 
 pub struct WorldRef;
 pub struct WorldMut;
-pub struct Comp<T>(PhantomData<T>);
+pub struct Comp<T>(PhantomData<fn() -> T>);
 
 #[derive(Default)]
 pub struct Multiverse {
     pub parallel: bool,
-    universes: HashMap<UniverseID, Universe>,
-    pipelines: HashMap<PipelineID, Box<dyn PipelineEngine + Send + Sync>>,
-    bindings: HashMap<UniverseID, PipelineID>,
-    default_universe: Option<UniverseID>,
+    universes: HashMap<UniverseId, Universe>,
+    pipelines: HashMap<PipelineId, Box<dyn PipelineEngine + Send + Sync>>,
+    bindings: HashMap<UniverseId, PipelineId>,
+    default_universe: Option<UniverseId>,
 }
 
 impl Multiverse {
@@ -58,16 +58,16 @@ impl Multiverse {
         self
     }
 
-    pub fn create_universe<S>(&mut self, state: S) -> UniverseID
+    pub fn create_universe<S>(&mut self, state: S) -> UniverseId
     where
         S: State + 'static,
     {
-        let id = UniverseID::new();
+        let id = UniverseId::new();
         self.universes.insert(id, Universe::new(state));
         id
     }
 
-    pub fn delete_universe(&mut self, id: UniverseID) -> Option<Universe> {
+    pub fn delete_universe(&mut self, id: UniverseId) -> Option<Universe> {
         if let Some(uni) = self.default_universe {
             if uni == id {
                 self.default_universe = None;
@@ -77,11 +77,11 @@ impl Multiverse {
         self.universes.remove(&id)
     }
 
-    pub fn default_universe_id(&self) -> Option<UniverseID> {
+    pub fn default_universe_id(&self) -> Option<UniverseId> {
         self.default_universe
     }
 
-    pub fn set_default_universe_id(&mut self, id: Option<UniverseID>) {
+    pub fn set_default_universe_id(&mut self, id: Option<UniverseId>) {
         self.default_universe = id;
     }
 
@@ -101,15 +101,15 @@ impl Multiverse {
         }
     }
 
-    pub fn universe(&self, id: UniverseID) -> Option<&Universe> {
+    pub fn universe(&self, id: UniverseId) -> Option<&Universe> {
         self.universes.get(&id)
     }
 
-    pub fn universe_mut(&mut self, id: UniverseID) -> Option<&mut Universe> {
+    pub fn universe_mut(&mut self, id: UniverseId) -> Option<&mut Universe> {
         self.universes.get_mut(&id)
     }
 
-    pub fn universe_ids(&self) -> impl Iterator<Item = UniverseID> + '_ {
+    pub fn universe_ids(&self) -> impl Iterator<Item = UniverseId> + '_ {
         self.universes.keys().cloned()
     }
 
@@ -121,37 +121,37 @@ impl Multiverse {
         self.universes.values_mut()
     }
 
-    pub fn universes_with_ids(&self) -> impl Iterator<Item = (UniverseID, &Universe)> {
+    pub fn universes_with_ids(&self) -> impl Iterator<Item = (UniverseId, &Universe)> {
         self.universes.iter().map(|(id, u)| (*id, u))
     }
 
-    pub fn universes_with_ids_mut(&mut self) -> impl Iterator<Item = (UniverseID, &mut Universe)> {
+    pub fn universes_with_ids_mut(&mut self) -> impl Iterator<Item = (UniverseId, &mut Universe)> {
         self.universes.iter_mut().map(|(id, u)| (*id, u))
     }
 
-    pub fn insert_pipeline<T>(&mut self, pipeline: T) -> PipelineID
+    pub fn insert_pipeline<T>(&mut self, pipeline: T) -> PipelineId
     where
         T: PipelineEngine + 'static + Send + Sync,
     {
-        let id = PipelineID::new();
+        let id = PipelineId::new();
         self.pipelines.insert(id, Box::new(pipeline));
         id
     }
 
-    pub fn remove_pipeline(&mut self, id: PipelineID) {
+    pub fn remove_pipeline(&mut self, id: PipelineId) {
         self.bindings.retain(|_, p| p != &id);
         self.pipelines.remove(&id);
     }
 
-    pub fn pipeline_ids(&self) -> impl Iterator<Item = PipelineID> + '_ {
+    pub fn pipeline_ids(&self) -> impl Iterator<Item = PipelineId> + '_ {
         self.pipelines.keys().cloned()
     }
 
-    pub fn bind(&mut self, universe: UniverseID, pipeline: PipelineID) {
+    pub fn bind(&mut self, universe: UniverseId, pipeline: PipelineId) {
         self.bindings.insert(universe, pipeline);
     }
 
-    pub fn unbind(&mut self, universe: UniverseID) {
+    pub fn unbind(&mut self, universe: UniverseId) {
         self.bindings.remove(&universe);
     }
 
@@ -572,7 +572,7 @@ pub trait ResQuery {
 
 pub struct ResRead<T> {
     inner: Ref<'static, Box<Resource>>,
-    _phantom: PhantomData<T>,
+    _phantom: PhantomData<fn() -> T>,
 }
 
 impl<T> ResAccess for ResRead<T> {}
@@ -604,7 +604,7 @@ impl<T> Deref for RefRead<T> {
 
 pub struct ResWrite<T> {
     inner: RefMut<'static, Box<Resource>>,
-    _phantom: PhantomData<T>,
+    _phantom: PhantomData<fn() -> T>,
 }
 
 impl<T> ResAccess for ResWrite<T> {}
@@ -667,6 +667,12 @@ impl ResQuery for WorldMut {
     }
 }
 
+impl ResQuery for () {
+    type Fetch = ();
+
+    fn fetch(_: &Universe) -> Self::Fetch {}
+}
+
 impl<T> ResQuery for Comp<T> {
     type Fetch = ();
 
