@@ -5,7 +5,7 @@ pub use hecs::*;
 use std::{any::TypeId, collections::HashSet, marker::PhantomData};
 use typid::ID;
 
-pub type PipelineID = ID<PhantomData<dyn PipelineEngine + Send + Sync>>;
+pub type PipelineId = ID<PhantomData<dyn PipelineEngine + Send + Sync>>;
 
 #[derive(Debug, Clone, PartialEq)]
 pub enum PipelineBuilderError {
@@ -32,6 +32,7 @@ pub trait PipelineBuilder: Sized {
         system: System,
         dependencies: &[&str],
         layer: PipelineLayer,
+        lock_on_single_thread: bool,
     ) -> Result<(), PipelineBuilderError>;
 
     fn add_system<AT: AccessType>(
@@ -40,7 +41,16 @@ pub trait PipelineBuilder: Sized {
         system: System,
         dependencies: &[&str],
     ) -> Result<(), PipelineBuilderError> {
-        self.add_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main)
+        self.add_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main, false)
+    }
+
+    fn add_system_on_single_thread<AT: AccessType>(
+        &mut self,
+        name: &str,
+        system: System,
+        dependencies: &[&str],
+    ) -> Result<(), PipelineBuilderError> {
+        self.add_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main, true)
     }
 
     fn with_system_on_layer<AT: AccessType>(
@@ -49,8 +59,9 @@ pub trait PipelineBuilder: Sized {
         system: System,
         dependencies: &[&str],
         layer: PipelineLayer,
+        lock_on_single_thread: bool,
     ) -> Result<Self, PipelineBuilderError> {
-        self.add_system_on_layer::<AT>(name, system, dependencies, layer)?;
+        self.add_system_on_layer::<AT>(name, system, dependencies, layer, lock_on_single_thread)?;
         Ok(self)
     }
 
@@ -60,7 +71,16 @@ pub trait PipelineBuilder: Sized {
         system: System,
         dependencies: &[&str],
     ) -> Result<Self, PipelineBuilderError> {
-        self.with_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main)
+        self.with_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main, false)
+    }
+
+    fn with_system_on_single_thread<AT: AccessType>(
+        self,
+        name: &str,
+        system: System,
+        dependencies: &[&str],
+    ) -> Result<Self, PipelineBuilderError> {
+        self.with_system_on_layer::<AT>(name, system, dependencies, PipelineLayer::Main, true)
     }
 
     fn graph(self) -> PipelineGraph;
@@ -125,6 +145,7 @@ impl PipelineBuilder for ParallelPipelineBuilder {
         system: System,
         dependencies: &[&str],
         layer: PipelineLayer,
+        lock_on_single_thread: bool,
     ) -> Result<(), PipelineBuilderError> {
         let systems = match layer {
             PipelineLayer::Pre => &mut self.systems_pre,
@@ -148,6 +169,7 @@ impl PipelineBuilder for ParallelPipelineBuilder {
                     reads,
                     writes,
                     layer,
+                    lock_on_single_thread,
                 },
             }]);
             return Ok(());
@@ -170,6 +192,7 @@ impl PipelineBuilder for ParallelPipelineBuilder {
                         reads,
                         writes,
                         layer,
+                        lock_on_single_thread,
                     },
                 });
                 return Ok(());
@@ -182,6 +205,7 @@ impl PipelineBuilder for ParallelPipelineBuilder {
                 reads,
                 writes,
                 layer,
+                lock_on_single_thread,
             },
         }]);
         Ok(())
@@ -234,6 +258,7 @@ impl PipelineBuilder for LinearPipelineBuilder {
         system: System,
         dependencies: &[&str],
         layer: PipelineLayer,
+        lock_on_single_thread: bool,
     ) -> Result<(), PipelineBuilderError> {
         let systems = match layer {
             PipelineLayer::Pre => &mut self.systems_pre,
@@ -253,6 +278,7 @@ impl PipelineBuilder for LinearPipelineBuilder {
                 reads,
                 writes,
                 layer,
+                lock_on_single_thread,
             },
         });
         Ok(())
@@ -284,6 +310,7 @@ pub struct PipelineGraphSystem {
     pub reads: HashSet<TypeId>,
     pub writes: HashSet<TypeId>,
     pub layer: PipelineLayer,
+    pub lock_on_single_thread: bool,
 }
 
 impl PartialEq for PipelineGraphSystem {
@@ -300,6 +327,7 @@ impl std::fmt::Debug for PipelineGraphSystem {
             .field("system", &format!("{:p}", self.system as *const ()))
             .field("reads", &self.reads)
             .field("writes", &self.writes)
+            .field("lock_on_single_thread", &self.lock_on_single_thread)
             .finish()
     }
 }
@@ -311,6 +339,18 @@ pub enum PipelineGraph {
     Parallel(Vec<PipelineGraph>),
 }
 
+impl PipelineGraph {
+    pub fn is_lock_on_single_thread(&self) -> bool {
+        matches!(
+            self,
+            Self::System(PipelineGraphSystem {
+                lock_on_single_thread: true,
+                ..
+            })
+        )
+    }
+}
+
 pub trait PipelineEngine {
     fn setup(&mut self, graph: PipelineGraph);
     fn run(&self, universe: &mut Universe);
@@ -371,6 +411,7 @@ mod tests {
                                 reads: types!(),
                                 writes: types!(A),
                                 layer: PipelineLayer::Main,
+                                lock_on_single_thread: false,
                             },
                         },
                         PipelineBuilderMeta {
@@ -380,6 +421,7 @@ mod tests {
                                 reads: types!(),
                                 writes: types!(B),
                                 layer: PipelineLayer::Main,
+                                lock_on_single_thread: false,
                             },
                         },
                         PipelineBuilderMeta {
@@ -389,6 +431,7 @@ mod tests {
                                 reads: types!(),
                                 writes: types!(),
                                 layer: PipelineLayer::Main,
+                                lock_on_single_thread: false,
                             },
                         },
                     ],
@@ -400,6 +443,7 @@ mod tests {
                                 reads: types!(),
                                 writes: types!(A, B),
                                 layer: PipelineLayer::Main,
+                                lock_on_single_thread: false,
                             },
                         },
                         PipelineBuilderMeta {
@@ -409,6 +453,7 @@ mod tests {
                                 reads: types!(),
                                 writes: types!(C),
                                 layer: PipelineLayer::Main,
+                                lock_on_single_thread: false,
                             },
                         },
                     ],
@@ -425,18 +470,21 @@ mod tests {
                         reads: types!(),
                         writes: types!(A),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_b,
                         reads: types!(),
                         writes: types!(B),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_c,
                         reads: types!(),
                         writes: types!(),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                 ]),
                 PipelineGraph::Parallel(vec![
@@ -445,12 +493,14 @@ mod tests {
                         reads: types!(),
                         writes: types!(A, B),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_c,
                         reads: types!(),
                         writes: types!(C),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                 ]),
             ])
@@ -472,18 +522,21 @@ mod tests {
                             reads: types!(),
                             writes: types!(A),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         }),
                         PipelineGraph::System(PipelineGraphSystem {
                             system: system_b,
                             reads: types!(),
                             writes: types!(B),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         }),
                         PipelineGraph::System(PipelineGraphSystem {
                             system: system_c,
                             reads: types!(),
                             writes: types!(),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         }),
                     ]),
                     PipelineGraph::Parallel(vec![
@@ -492,12 +545,14 @@ mod tests {
                             reads: types!(),
                             writes: types!(A, B),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         }),
                         PipelineGraph::System(PipelineGraphSystem {
                             system: system_c,
                             reads: types!(),
                             writes: types!(C),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         }),
                     ]),
                 ])),
@@ -527,6 +582,7 @@ mod tests {
                             reads: types!(),
                             writes: types!(A),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         },
                     },
                     PipelineBuilderMeta {
@@ -536,6 +592,7 @@ mod tests {
                             reads: types!(),
                             writes: types!(B),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         },
                     },
                     PipelineBuilderMeta {
@@ -545,6 +602,7 @@ mod tests {
                             reads: types!(),
                             writes: types!(A, B),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         },
                     },
                     PipelineBuilderMeta {
@@ -554,6 +612,7 @@ mod tests {
                             reads: types!(),
                             writes: types!(C),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         },
                     },
                     PipelineBuilderMeta {
@@ -563,6 +622,7 @@ mod tests {
                             reads: types!(),
                             writes: types!(),
                             layer: PipelineLayer::Main,
+                            lock_on_single_thread: false,
                         },
                     },
                 ],
@@ -577,30 +637,35 @@ mod tests {
                     reads: types!(),
                     writes: types!(A),
                     layer: PipelineLayer::Main,
+                    lock_on_single_thread: false,
                 }),
                 PipelineGraph::System(PipelineGraphSystem {
                     system: system_b,
                     reads: types!(),
                     writes: types!(B),
                     layer: PipelineLayer::Main,
+                    lock_on_single_thread: false,
                 }),
                 PipelineGraph::System(PipelineGraphSystem {
                     system: system_c,
                     reads: types!(),
                     writes: types!(A, B),
                     layer: PipelineLayer::Main,
+                    lock_on_single_thread: false,
                 }),
                 PipelineGraph::System(PipelineGraphSystem {
                     system: system_c,
                     reads: types!(),
                     writes: types!(C),
                     layer: PipelineLayer::Main,
+                    lock_on_single_thread: false,
                 }),
                 PipelineGraph::System(PipelineGraphSystem {
                     system: system_c,
                     reads: types!(),
                     writes: types!(),
                     layer: PipelineLayer::Main,
+                    lock_on_single_thread: false,
                 }),
             ])
         );
@@ -620,30 +685,35 @@ mod tests {
                         reads: types!(),
                         writes: types!(A),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_b,
                         reads: types!(),
                         writes: types!(B),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_c,
                         reads: types!(),
                         writes: types!(A, B),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_c,
                         reads: types!(),
                         writes: types!(C),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                     PipelineGraph::System(PipelineGraphSystem {
                         system: system_c,
                         reads: types!(),
                         writes: types!(),
                         layer: PipelineLayer::Main,
+                        lock_on_single_thread: false,
                     }),
                 ])),
             }
